@@ -35,28 +35,25 @@ QString DbInitializerMySql::sqlType(const QString & type, int size) const
     return DbInitializer::sqlType( type, size );
 }
 
-QString DbInitializerMySql::hasIndexQuery(const QString& tableName, const QString& indexName)
-{
-  return QString::fromLatin1( "SHOW INDEXES FROM %1 WHERE `Key_name` = '%2'" )
-      .arg( tableName ).arg( indexName );
-}
-
 QString DbInitializerMySql::buildCreateTableStatement( const TableDescription &tableDescription ) const
 {
   QStringList columns;
   QStringList references;
 
   Q_FOREACH ( const ColumnDescription &columnDescription, tableDescription.columns ) {
-    columns.append( buildColumnStatement( columnDescription ) );
+    columns.append( buildColumnStatement( columnDescription, tableDescription ) );
 
     if ( !columnDescription.refTable.isEmpty() && !columnDescription.refColumn.isEmpty() ) {
-      references << QString::fromLatin1( "FOREIGN KEY (%1) REFERENCES %2Table(%3) ON DELETE CASCADE ON UPDATE CASCADE" )
+      references << QString::fromLatin1( "FOREIGN KEY (%1) REFERENCES %2Table(%3) " )
                                        .arg( columnDescription.name )
                                        .arg( columnDescription.refTable )
-                                       .arg( columnDescription.refColumn );
+                                       .arg( columnDescription.refColumn )
+                    + buildReferentialAction( columnDescription.onUpdate, columnDescription.onDelete );
     }
   }
 
+  if ( tableDescription.primaryKeyColumnCount() > 1 )
+    columns.push_back( buildPrimaryKeyStatement( tableDescription ) );
   columns << references;
 
   const QString tableProperties = QLatin1String( " COLLATE=utf8_general_ci DEFAULT CHARSET=utf8" );
@@ -64,7 +61,7 @@ QString DbInitializerMySql::buildCreateTableStatement( const TableDescription &t
   return QString::fromLatin1( "CREATE TABLE %1 (%2) %3" ).arg( tableDescription.name, columns.join( QLatin1String( ", " ) ), tableProperties );
 }
 
-QString DbInitializerMySql::buildColumnStatement( const ColumnDescription &columnDescription ) const
+QString DbInitializerMySql::buildColumnStatement( const ColumnDescription &columnDescription, const TableDescription &tableDescription ) const
 {
   QString column = columnDescription.name;
 
@@ -76,7 +73,7 @@ QString DbInitializerMySql::buildColumnStatement( const ColumnDescription &colum
   if ( columnDescription.isAutoIncrement )
     column += QLatin1String( " AUTO_INCREMENT" );
 
-  if ( columnDescription.isPrimaryKey )
+  if ( columnDescription.isPrimaryKey && tableDescription.primaryKeyColumnCount() == 1 )
     column += QLatin1String( " PRIMARY KEY" );
 
   if ( columnDescription.isUnique )
@@ -107,6 +104,18 @@ QString DbInitializerMySql::buildInsertValuesStatement( const TableDescription &
                             .arg( QStringList( data.values() ).join( QLatin1String( "," ) ) );
 }
 
+QString DbInitializerMySql::buildAddForeignKeyConstraintStatement(const DbInitializer::TableDescription& table, const DbInitializer::ColumnDescription& column) const
+{
+  return QLatin1Literal( "ALTER TABLE " ) + table.name + QLatin1Literal( " ADD FOREIGN KEY (" ) + column.name
+       + QLatin1Literal( ") REFERENCES " ) + column.refTable + QLatin1Literal( "Table(" ) + column.refColumn
+       + QLatin1Literal( ") " ) + buildReferentialAction( column.onUpdate, column.onDelete );
+}
+
+QString DbInitializerMySql::buildRemoveForeignKeyConstraintStatement(const DbIntrospector::ForeignKey& fk, const DbInitializer::TableDescription& table) const
+{
+  return QLatin1Literal( "ALTER TABLE " ) + table.name + QLatin1Literal( " DROP FOREIGN KEY " ) + fk.name;
+}
+
 //END MySQL
 
 
@@ -118,22 +127,20 @@ DbInitializerSqlite::DbInitializerSqlite(const QSqlDatabase& database, const QSt
 {
 }
 
-QString DbInitializerSqlite::hasIndexQuery(const QString& tableName, const QString& indexName)
-{
-  return QString::fromLatin1( "SELECT * FROM sqlite_master WHERE type='index' AND tbl_name='%1' AND name='%2';" ).arg( tableName ).arg( indexName );
-}
-
 QString DbInitializerSqlite::buildCreateTableStatement( const TableDescription &tableDescription ) const
 {
   QStringList columns;
 
   Q_FOREACH ( const ColumnDescription &columnDescription, tableDescription.columns )
-    columns.append( buildColumnStatement( columnDescription ) );
+    columns.append( buildColumnStatement( columnDescription, tableDescription ) );
+
+  if ( tableDescription.primaryKeyColumnCount() > 1 )
+    columns.push_back( buildPrimaryKeyStatement( tableDescription ) );
 
   return QString::fromLatin1( "CREATE TABLE %1 (%2)" ).arg( tableDescription.name, columns.join( QLatin1String( ", " ) ) );
 }
 
-QString DbInitializerSqlite::buildColumnStatement( const ColumnDescription &columnDescription ) const
+QString DbInitializerSqlite::buildColumnStatement( const ColumnDescription &columnDescription, const TableDescription &tableDescription ) const
 {
   QString column = columnDescription.name + QLatin1Char( ' ' );
 
@@ -142,7 +149,7 @@ QString DbInitializerSqlite::buildColumnStatement( const ColumnDescription &colu
   else
     column += sqlType( columnDescription.type, columnDescription.size );
 
-  if ( columnDescription.isPrimaryKey )
+  if ( columnDescription.isPrimaryKey && tableDescription.primaryKeyColumnCount() == 1 )
     column += QLatin1String( " PRIMARY KEY" );
   else if ( columnDescription.isUnique )
     column += QLatin1String( " UNIQUE" );
@@ -201,25 +208,20 @@ QString DbInitializerPostgreSql::sqlType(const QString& type, int size) const
   return DbInitializer::sqlType( type, size );
 }
 
-QString DbInitializerPostgreSql::hasIndexQuery(const QString& tableName, const QString& indexName)
-{
-  QString query = QLatin1String( "SELECT indexname FROM pg_catalog.pg_indexes" );
-  query += QString::fromLatin1( " WHERE tablename ilike '%1'" ).arg( tableName );
-  query += QString::fromLatin1( " AND  indexname ilike '%1';" ).arg( indexName );
-  return query;
-}
-
 QString DbInitializerPostgreSql::buildCreateTableStatement( const TableDescription &tableDescription ) const
 {
   QStringList columns;
 
   Q_FOREACH ( const ColumnDescription &columnDescription, tableDescription.columns )
-    columns.append( buildColumnStatement( columnDescription ) );
+    columns.append( buildColumnStatement( columnDescription, tableDescription ) );
+
+  if ( tableDescription.primaryKeyColumnCount() > 1 )
+    columns.push_back( buildPrimaryKeyStatement( tableDescription ) );
 
   return QString::fromLatin1( "CREATE TABLE %1 (%2)" ).arg( tableDescription.name, columns.join( QLatin1String( ", " ) ) );
 }
 
-QString DbInitializerPostgreSql::buildColumnStatement( const ColumnDescription &columnDescription ) const
+QString DbInitializerPostgreSql::buildColumnStatement( const ColumnDescription &columnDescription, const TableDescription &tableDescription ) const
 {
   QString column = columnDescription.name + QLatin1Char( ' ' );
 
@@ -228,18 +230,19 @@ QString DbInitializerPostgreSql::buildColumnStatement( const ColumnDescription &
   else
     column += sqlType( columnDescription.type, columnDescription.size );
 
-  if ( columnDescription.isPrimaryKey )
+  if ( columnDescription.isPrimaryKey && tableDescription.primaryKeyColumnCount() == 1 )
     column += QLatin1String( " PRIMARY KEY" );
   else if ( columnDescription.isUnique )
     column += QLatin1String( " UNIQUE" );
 
-  if ( !columnDescription.allowNull && !columnDescription.isPrimaryKey )
+  if ( !columnDescription.allowNull && !( columnDescription.isPrimaryKey && tableDescription.primaryKeyColumnCount() == 1 ) )
     column += QLatin1String( " NOT NULL" );
 
   if ( !columnDescription.refTable.isEmpty() && !columnDescription.refColumn.isEmpty() ) {
-    column += QString::fromLatin1( " REFERENCES %1Table(%2) ON DELETE CASCADE ON UPDATE CASCADE" )
+    column += QString::fromLatin1( " REFERENCES %1Table(%2) " )
                                  .arg( columnDescription.refTable )
-                                 .arg( columnDescription.refColumn );
+                                 .arg( columnDescription.refColumn )
+           +  buildReferentialAction( columnDescription.onUpdate, columnDescription.onDelete );
   }
 
   if ( !columnDescription.defaultValue.isEmpty() ) {
@@ -267,35 +270,6 @@ QString DbInitializerPostgreSql::buildInsertValuesStatement( const TableDescript
                             .arg( QStringList( data.values() ).join( QLatin1String( "," ) ) );
 }
 
-QString DbInitializerPostgreSql::buildCreateRelationTableStatement( const QString &tableName, const RelationDescription &relationDescription ) const
-{
-  const QString columnOptions = QLatin1String( " ON DELETE CASCADE ON UPDATE CASCADE" );
-
-  QString statement = QString::fromLatin1( "CREATE TABLE %1 (" ).arg( tableName );
-
-  statement += QString::fromLatin1( "%1_%2 INTEGER REFERENCES %3(%4) %5, " )
-      .arg( relationDescription.firstTable )
-      .arg( relationDescription.firstColumn )
-      .arg( relationDescription.firstTableName )
-      .arg( relationDescription.firstColumn )
-      .arg( columnOptions );
-
-  statement += QString::fromLatin1( "%1_%2 INTEGER REFERENCES %3(%4) %5, " )
-      .arg( relationDescription.secondTable )
-      .arg( relationDescription.secondColumn )
-      .arg( relationDescription.secondTableName )
-      .arg( relationDescription.secondColumn )
-      .arg( columnOptions );
-
-  statement += QString::fromLatin1( "PRIMARY KEY (%1_%2, %3_%4))" )
-      .arg( relationDescription.firstTable )
-      .arg( relationDescription.firstColumn )
-      .arg( relationDescription.secondTable )
-      .arg( relationDescription.secondColumn );
-
-  return statement;
-}
-
 //END PostgreSQL
 
 
@@ -327,23 +301,20 @@ QString DbInitializerVirtuoso::sqlValue(const QString& type, const QString& valu
   return DbInitializer::sqlValue( type, value );
 }
 
-bool DbInitializerVirtuoso::hasIndex(const QString& tableName, const QString& indexName)
-{
-  // TODO: Implement index checking for Virtuoso!
-  return true;
-}
-
 QString DbInitializerVirtuoso::buildCreateTableStatement( const TableDescription &tableDescription ) const
 {
   QStringList columns;
 
   Q_FOREACH ( const ColumnDescription &columnDescription, tableDescription.columns )
-    columns.append( buildColumnStatement( columnDescription ) );
+    columns.append( buildColumnStatement( columnDescription, tableDescription ) );
+
+  if ( tableDescription.primaryKeyColumnCount() > 1 )
+    columns.push_back( buildPrimaryKeyStatement( tableDescription ) );
 
   return QString::fromLatin1( "CREATE TABLE %1 (%2)" ).arg( tableDescription.name, columns.join( QLatin1String( ", " ) ) );
 }
 
-QString DbInitializerVirtuoso::buildColumnStatement( const ColumnDescription &columnDescription ) const
+QString DbInitializerVirtuoso::buildColumnStatement( const ColumnDescription &columnDescription, const TableDescription &tableDescription ) const
 {
   QString column = columnDescription.name;
 
@@ -355,16 +326,15 @@ QString DbInitializerVirtuoso::buildColumnStatement( const ColumnDescription &co
   if ( columnDescription.isAutoIncrement )
     column += QLatin1String( " IDENTITY" );
 
-  if ( columnDescription.isPrimaryKey )
+  if ( columnDescription.isPrimaryKey && tableDescription.primaryKeyColumnCount() == 1 )
     column += QLatin1String( " PRIMARY KEY" );
 
   if ( columnDescription.isUnique )
     column += QLatin1String( " UNIQUE" );
 
   if ( !columnDescription.refTable.isEmpty() && !columnDescription.refColumn.isEmpty() ) {
-    column += QString::fromLatin1( " REFERENCES %1Table(%2) ON DELETE CASCADE ON UPDATE CASCADE" )
-                                 .arg( columnDescription.refTable )
-                                 .arg( columnDescription.refColumn );
+    column += QLatin1Literal( " REFERENCES " ) + columnDescription.refTable + QLatin1Literal( "Table(" )
+           +  columnDescription.refColumn + QLatin1Literal( ") " ) + buildReferentialAction( columnDescription.onUpdate, columnDescription.onDelete );
   }
 
   if ( !columnDescription.defaultValue.isEmpty() ) {
